//---------------------------------------------------------------------------
//  watek odczytu danych z USB

#include "ThreadComm.h"
#include <stdio.h>
#include "..\miniscope\device\MiniscopeDevice.h"
#include "ScopedLock.h"

#pragma package(smart_init)

bool writeLock;

__fastcall ThreadComm::ThreadComm()
		: callbackLog(NULL), callbackDataRcv(NULL),
		connected(true)
{
}
//---------------------------------------------------------------------------


void ThreadComm::log(char *lpData, ...)
{
    static char buf[256]; //determines max message length
	va_list ap;
	va_start(ap, lpData);
	vsnprintf(buf, sizeof(buf), lpData, ap);
	va_end(ap);
	buf[sizeof(buf)-1] = 0;

	if (callbackLog)
		callbackLog(buf);
}

static DWORD ReadDataFromBuffer(LPDIRECTSOUNDCAPTUREBUFFER lpDsb, // The buffer.
				  DWORD dwOffset,		    // Our own write cursor.
				  LPBYTE lpbSoundData,		    // Start of our data.
				  DWORD dwSoundBytes)		    // Size of block to copy.
{ 
    LPVOID  lpvPtr1; 
    DWORD dwBytes1; 
    LPVOID  lpvPtr2; 
    DWORD dwBytes2; 
    HRESULT hr; 
    
    // Obtain memory address of write block. This will be in two parts
    // if the block wraps around.
    
    hr = IDirectSoundCaptureBuffer_Lock( lpDsb, dwOffset, dwSoundBytes, &lpvPtr1, 
					 &dwBytes1, &lpvPtr2, &dwBytes2, 0); 
    
    if SUCCEEDED(hr) { 
	// Read from pointers. 
	memcpy(lpbSoundData, lpvPtr1, dwBytes1);
	if (lpvPtr2 != NULL)
	    memcpy(lpbSoundData+dwBytes1, lpvPtr2, dwBytes2); 
	
	// Release the data back to DirectSound. 
	hr = IDirectSoundCaptureBuffer_Unlock(lpDsb, lpvPtr1, dwBytes1, lpvPtr2, dwBytes2); 
	if SUCCEEDED(hr)
		return 1;
    } 
    
    // Lock, Unlock, or Restore failed. 
    return 0; 
}

/*
 * Check if there are captured frames in DirectSound capture buffer.
 */
static unsigned dsound_captured_size(ThreadComm::DsoundStream* stream)
{
    HRESULT hr;
    long size_available;
    DWORD writePos, readPos;

	hr = IDirectSoundCaptureBuffer_GetCurrentPosition(stream->lpDsBuffer, &writePos, &readPos);
	if FAILED(hr)
		return 0;

	if (readPos < stream->dwBytePos)
		size_available = readPos + (stream->dwDsBufferSize) - stream->dwBytePos;
	else
		size_available = readPos - stream->dwBytePos;

    return size_available;
}

//---------------------------------------------------------------------------
DWORD WINAPI ThreadCommProc(LPVOID data)
{
	ThreadComm *thrRead = (ThreadComm*)data;
	ThreadComm::DsoundStream *stream = thrRead->stream;

	while (thrRead->connected)
	{
		DWORD rc;
		rc = WaitForSingleObject(stream->hEvent, 1000);
		if (rc != WAIT_OBJECT_0)
			continue;

		/*
		 * DirectSound has indicated that it has some frames ready
		 * in the capture buffer. Get as much frames as possible to
		 * prevent overflows.
		 */
		do {
			/* Capture from DirectSound buffer. */
			rc = ReadDataFromBuffer(stream->lpDsBuffer,
						   stream->dwBytePos,
						   (LPBYTE)stream->buffer,
						   stream->bytes_per_frame);

			if (!rc) {
				break;
			}

			/* Call callback */
			thrRead->callbackDataRcv((short*)stream->buffer, stream->bytes_per_frame/sizeof(short));

			/* Increment position. */
			stream->dwBytePos += stream->bytes_per_frame;
			if (stream->dwBytePos >= stream->dwDsBufferSize)
				stream->dwBytePos -= stream->dwDsBufferSize;

			/* Fetch while we have more than 1 frame */
		} while (dsound_captured_size(stream) > stream->bytes_per_frame);

	}

	thrRead->connected = false;
	return 0;
}


void ThreadComm::Execute(void)
{
	DWORD dwtid;
	Thread = CreateThread(NULL, 0, ThreadCommProc, this, 0, &dwtid);
	if (Thread)
	{
		bool status = SetThreadPriority(Thread, THREAD_PRIORITY_TIME_CRITICAL);
		if (!status)
			log("Failed to set communication thread priority!");
	}
	else
	{
		log("Failed to create communication thread!");
	}
}


